import Qt 4.7
import QtWebKit 1.0

import "imports/XAComponents"
import "imports/WebBrowserContent"
import "imports/colibri"

Item {
    id: attachmentBrowser

    property string urlString : qtInputURL
    property string windowTitle : qtInputTitle
    property string windowGotFocus : qtWindowGotFocus
    property string platformID: qtPlatformID

    anchors.fill: parent

    focus: true
    Keys.enabled: true
    Keys.onSpacePressed: closeWindow();

    onWindowTitleChanged: {
//        console.debug("<< onWindowTitleChanged")
        attachmentBrowser.focus = true
        Keys.enabled = true
    }

    function loadFirstAttachment(uid, url, att_type) {
        if (attachmentGrid.count == 0 || uid === "") {
            previewArea.emptyPreview();
            attachmentBrowserRect.color = "#99cbcbcb"

        } else {
            attachmentGrid.currentIndex = 0;
            previewArea.loadPreview (uid, url, att_type);
        }
    }

    function resetBrowserOnHide() {
        attachmentGrid.currentIndex = -1;
        urlString = "";
    }

    function closeWindow() {
//        console.debug("<< closeWindow()")

        QtHandler.CloseButtonClicked();
    }

    Rectangle {
        id: windowRect
        opacity: 0.9
        color: "#eeffffff"

        radius: 6
        smooth: true
        anchors.fill: parent
        anchors.rightMargin: (attachmentBrowser.platformID == "WIN" ? 1 : 0)
        anchors.bottomMargin: (attachmentBrowser.platformID == "WIN" ? 1 : 0)

        //clip: true

        //
        // window title bar
        //

        XAWindowTitleBarWhite {
            id: xaWindowTitleBar
            windowTitle: attachmentBrowser.windowTitle
            windowGotFocus: attachmentBrowser.windowGotFocus
            windowDrawBorder: (attachmentBrowser.platformID == "WIN")
            platformID: attachmentBrowser.platformID
        }

        //
        // attachment selection bar
        //

        Rectangle {
            id: attachmentBrowserRect
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.rightMargin: (attachmentBrowser.platformID == "WIN" ? 1 : 0)
            anchors.leftMargin: (attachmentBrowser.platformID == "WIN" ? 1 : 0)
            anchors.top: parent.top
            anchors.topMargin: 19
            height: 74

            clip: true
            opacity: 1.0
            color:  "#22222222"

            GridView {
                id: attachmentGrid
                anchors.fill: parent
                anchors.topMargin: 2
                anchors.leftMargin: 2
                anchors.rightMargin: 2

                cellWidth: 80; cellHeight: 70

                Component {
                    id: attachmentBrowserDelegate

                    AIAttachmentsBrowserDelegate {
                        width: attachmentGrid.cellWidth; height: attachmentGrid.cellHeight
                    }
                }

                model: attachmentModel
                delegate: attachmentBrowserDelegate

                highlight: Item {
                    Rectangle {
                        // icon background on selection
                        color: "#cbcbcb";
                        radius: 8;
                        smooth: true;
                        anchors.top: parent.top; anchors.bottom: parent.bottom; anchors.bottomMargin: 15
                        anchors.left: parent.left; anchors.right: parent.right

                    }
                    Rectangle {
                        // text background on selection
                        color: "#3875d7";
                        radius: 8;
                        smooth: true;
                        anchors.left: parent.left; anchors.right: parent.right; anchors.bottom: parent.bottom
                        height: 14
                    }
                }
                focus: true

                onModelChanged: {
//                    console.debug("<< New Model set for AttachmentGrid")

                    if (attachmentGrid.count == 0) {
                        previewArea.emptyPreview();
                        attachmentBrowserRect.color = "#99cbcbcb"

                    } else {
//                        console.debug("<< New Model contains " + attachmentGrid.count + " items.")
                        attachmentGrid.currentIndex = -1;
                        previewArea.deselectedPreviewItem();
                        //attachmentGrid.currentIndex = 0;
                        //previewArea.loadCurrentItemPreview();
                        //previewArea.loadPreview(currentItem.uid, currentItem.url, currentItem.att_type)
                        //previewArea.loadPreview(currentItem.uid, "http://www.taskfabric.com", currentItem.att_type)
                    }
                    //previewArea.loadPreview(currentItem.uid, "http://www.taskfabric.com", currentItem.att_type)
                    //previewArea.loadCurrentItemPreview();
                }

                flickableChildren: MouseArea {
                    id: attachmentBrowserBackgroundArea
                    anchors.fill: parent

                    onClicked: {
//                        console.debug("<< Background Click detected")
                        attachmentGrid.currentIndex = -1;
                        previewArea.deselectedPreviewItem();
                    }
                }
            }
        }

    }

    //
    // preview area
    //

    Rectangle {
        id: previewArea
        anchors.right: parent.right
        anchors.left: parent.left
        anchors.rightMargin: (attachmentBrowser.platformID == "WIN" ? 1 : 0)
        anchors.leftMargin: (attachmentBrowser.platformID == "WIN" ? 1 : 0)
        anchors.top: parent.top
        anchors.topMargin: 94
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 19

        function deselectedPreviewItem() {
            previewArea.loadCurrentItemPreview();
            attachmentBrowserRect.color = "#22222222"
            placeHolder.text = qsTr("Select Attachment for Preview.\nDouble-Click to open.")
        }

        function emptyPreview() {
            //console.debug("QML::emptyPreview previewArea emptyPreview()")

            webBrowser.visible = false;
            imageBrowser.visible = false;
            attachmentBrowser.urlString = "about:blank";

            placeHolderFrame.visible = true;
            placeHolder.text = qsTr("No Attachments available.")
        }

        function loadCurrentItemPreview() {
            //console.debug("QML::loadCurrentItemPreview previewArea loadCurrentItemPreview()")

            if (attachmentGrid.currentItem != null) {
//                console.debug(attachmentGrid.currentItem.delegateData.uid)
//                console.debug(attachmentGrid.currentItem.uid)
//                console.debug(attachmentGrid.currentItem.url)
//                console.debug(attachmentGrid.currentItem.att_type)

                loadPreview(attachmentGrid.currentItem.uid, attachmentGrid.currentItem.url, attachmentGrid.currentItem.att_type)
            } else {
                emptyPreview();
            }
        }

        function loadPreview (uid, url, att_type) {
//            console.debug("<< previewArea loadPreview (uid, url, att_type): " + uid + "/" + url + "/" + att_type)

            placeHolderFrame.visible = false;
            webBrowser.visible = false;
            imageBrowser.visible = false;

            if (url === "" || url.substr(0,7) === "http://" || url.substr(0,8) === "https://") {
                webBrowser.visible = true;
                attachmentBrowser.urlString = url;
            } else {
                webBrowser.visible = false;
                attachmentBrowser.urlString = "about:blank";
            }

            if (att_type === "PictureMemo") {
                imageView.source = "";
                imageBrowser.visible = true;
                imageView.source = QtHandler.attachmentTempFileForUID(uid)

            } else if (att_type === "FILE") {
                var ext3 = url.substr(url.length-3, 3).toLowerCase()
                var ext4 = url.substr(url.length-4, 4).toLowerCase()
                var ext6 = url.substr(url.length-6, 6).toLowerCase()
//                console.debug("QML++ AIATTACHMENTSPREVIEW FILE TYPE:")
//                console.debug(ext3)
//                console.debug(ext4)

                if (ext3 === "jpg" || ext3 === "png" || ext3 === "gif" || ext4 === "jpeg" || ext3 === "tif" || ext4 === "tiff") {
                    imageView.source = "";
                    imageBrowser.visible = true;
                    imageView.source = QtHandler.attachmentTempFileForUID(uid)

                } else if (ext3 === "eml") {
                    webBrowser.visible = true;
                    attachmentBrowser.urlString = "about:blank";
                    webView.html = QtHandler.attachmentHTMLPreviewForUID(uid)

                } else if (ext6 === "webloc") {
                    webBrowser.visible = true;
                    attachmentBrowser.urlString = QtHandler.attachmentWeblocURLForUID(uid);

                } else {
                    placeHolderFrame.visible = true;
                    placeHolder.text = qsTr("No Preview available.")
                }
            }
        }

        Rectangle {
            id: placeHolderFrame
            anchors.fill: parent
            visible: false

            color: "#f0f0f0"

            Text {
                id: placeHolderShadow
                anchors.fill: parent
                anchors.topMargin: 2
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignHCenter

                font.family: "Helvetica Neue, Arial"
                font.pixelSize: 32
                font.bold: true

                text: placeHolder.text

                color: "white"
            }

            Text {
                id: placeHolder
                anchors.fill: parent
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignHCenter

                font.family: "Helvetica Neue, Arial"
                font.pixelSize: 32
                font.bold: true

                color: "#e0e0e0"
            }

        }

        Rectangle {
            id: imageBrowser
            anchors.fill: parent
            visible: false

            color: "#f0f0f0"

            Rectangle {
                id: imageBrowserDivisionLine
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: parent.top
                height: 1

                opacity: 0.2
                color: "gray"
            }

            Image {
                id: imageView
                anchors.fill: parent
                anchors.topMargin: 1
                asynchronous: true

                fillMode: Image.PreserveAspectFit
                smooth: true
            }
        }

        Rectangle {
            id: webBrowser
            anchors.fill: parent
            visible: true

            property int scrollbarwh: 12
            color: "#f0f0f0"

            Flickable {
                id: flickableWebView
                anchors { top: headerSpace.bottom; topMargin: 1; left: webBrowser.left; right: webBrowser.right; bottom: webBrowser.bottom; rightMargin: webBrowser.scrollbarwh }
                clip: true

                contentWidth: Math.max(parent.width,webView.width)
                contentHeight: Math.max(parent.height,webView.height)

                WebView {
                    id: webView
                    //anchors.horizontalCenter: parent.horizontalCenter

                    url: attachmentBrowser.urlString

                    settings.pluginsEnabled: true
                    settings.autoLoadImages: true
                    settings.javascriptEnabled: true

                    preferredWidth: flickableWebView.width
                    preferredHeight: flickableWebView.height
                    contentsScale: 1

                    onUrlChanged: {
                        // got to topleft
                        flickableWebView.contentX = 0
                        flickableWebView.contentY = 0
                        if (url != null) { header.editUrl = url.toString(); }
                    }
                    onProgressChanged: { header.urlChanged = false }
                    onLoadFinished: {
                        scroller1.updateSizing();
                    }

                }
            }

            Rectangle {
                id: headerSpace;
                anchors.top: parent.top
                width: parent.width; height: 38

                gradient: Gradient {
                    GradientStop { position: 0; color: "#222222" }
                    GradientStop { position: 0.5; color: "#3f3b3c" }
                    GradientStop { position: 1; color: "#222222" }
                }

                smooth: true

                Item {
                    width: headerSpace.width; height: 11
                    //color: "green"

                    Text {
                        anchors.centerIn: headerSpace.Center
                        anchors.top: parent.top
                        width: headerSpace.width; height: 11
                        text: webView.title; font.pixelSize: 9; font.bold: false
                        color: "white"; styleColor: "black";
                        smooth: true
                        verticalAlignment: Text.AlignVCenter
                        horizontalAlignment: Text.AlignHCenter
                    }
                }

                Header {
                    id: header
                    x: 0
                    editUrl: attachmentBrowser.urlString
                    width: headerSpace.width;
                    height: 26
                    anchors.top: parent.top
                    anchors.topMargin: 11
                }
            }

            XAScrollBar2 {
                id: scroller1
                scrollArea: flickableWebView; width: webBrowser.scrollbarwh
                anchors { right: parent.right; top: headerSpace.bottom; bottom: parent.bottom; }
            }
        }

    }


    //
    // bottom bar
    //

    Rectangle {
        id: bottombarRect
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: parent.bottom
        anchors.topMargin: -19
        height: 19

        color: "transparent"

        Rectangle {
            id: bottomBarDivisionLine
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.top: parent.top
            height: 1

            opacity: 0.2
            color: "gray"
        }

        MouseArea {
            id: bottombarDragArea
            anchors.fill: parent
            anchors.rightMargin: 15
            clip: false
            visible: true
            anchors.right: parent.right
            anchors.left: parent.left
            anchors.top: parent.top
            acceptedButtons: Qt.LeftButton

            onPressed: {
                QtHandler.startMoveTrigger(mouseX, mouseY)
            }
            onPositionChanged: {
                QtHandler.moveTrigger(mouseX, mouseY);
            }
        }

        Image {
            id: bottombarResizeImage
            //anchors.top: parent.top
            anchors.right: parent.right
            anchors.bottom: parent.bottom
            width: 15
            height: 15
            source: "imports/XAComponents/pics/PreviewCornerResizeHandle15px.png"

            MouseArea {
                id: bottombarResizeArea
                anchors.fill: parent
                clip: false
                visible: true
                acceptedButtons: Qt.LeftButton

                onPressed: {
                    QtHandler.startResizeTrigger(mouseX, mouseY)
                }
                onPositionChanged: {
                    QtHandler.resizeTrigger(mouseX, mouseY);
                }
            }
        }


    }
}
